Elasticsearch
----------------

1. [**Installation**](#h1-installation)
    1. [Installer Elasticsearch](#h2-installer-elastic)
    1. [Installer Kibana](#h2-installer-kibana)
    1. [Variables d'environnement](#h3-variables-environnement)
1. [**Premiers pas**](#h1-premiers-pas)
   1. [Comment faire les requêtes : Ligne de commande et console graphique de Kibana](#h1-comment-faire-requetes)
   1. [Vérifier si le serveur est accessible](#h1-verifier-serveur)
   1. [Création d'un index](#h1-creer-index)
   1. [Indexer un document](#h1-indexer-document)
   1. [Indexation massive](#h1-indexation-massive)
   1. [Effacer des documents](#h1-effacer-documents)
1. [**Les mappings : schéma de l'index**](#h1-mappings) 
    1. [Les mappings et y accéder](#h2-les-mappings-et-y-acceder)
    1. [Modification des mappings](#h2-modification-mappings)
    1. [Exemple](#h2-exemple-mappings)
1. [**Recherches**](#h1-recherches)
    1. [Recherches en texte intégral de base](#h2-recherches-texte-integral-basiques)
    1. [Recherches de base avec des requêtes JSON](#h2-recherches-basiques-en-json)
        1. [Requêtes de type `match`](#h3-requetes-match)
            1. [Changer l'opérateur de recherche](#h4-nombre-minimal-corresp)
            1. [Demander un nombre minimal de termes correspondants](#h4-recherches-avec-phrase)
            1. [Choisir les champs où chercher avec `multi_match`](#h4-multi-match)
        1. [Requêtes de type `term` : Recherche exacte et filtrage](#h3-recherches-exactes-et-filtrage)
    1. [Facettes](#h1-facettes)
        1. [Affichage de facettes](#h2-affichage-de-facettes)
1. [**Recherche sémantique avec embeddings par Mistral**](#h1-recherche-semantique)
   1. [Création d'un point d'accès dans ES qui communique avec l'API d'un modèle de langue](#h2-creation-point-acces)
   1. [Préparation de l'index pour stocker les embeddings](#h2-preparation-index-recherche-semantique)
   1. [Indexation](#h2-indexation-recherche-semantique)
   1. [Recherche](#h2-recherche-recherche-semantique)
1. [**Les dashboards dans Kibana**](#h1-visualisations-kibana)


Tout a été testé avec Ubuntu dans la salle 4s04 (Faculté des langues).

<a name="h1-installation"></a>
# Installation

Télécharger Elasticsearch (ES) et Kibana

- https://www.elastic.co/downloads/elasticsearch
- https://www.elastic.co/downloads/kibana

Dézippez les deux zip avant de continuer. 

<a name="h2-installer-elastic"></a>
## Installer Elasticsearch

```bash
./bin/elasticsearch
```

La première fois, il demandera de faire quelques configurations.

Notez les valeurs du mot de passe qui se génère ainsi que le token ("enrolement token"). Elles seront nécessaires pour :
- faire des requêtes vers ES
- utiliser l'interface Kibana.

S'il n'y a pas de problème, cela donnera une sortie comme la suivante, avec des informations pour connecter Kibana, qui sont valables pendant 30 minutes :

```
✅ Elasticsearch security features have been automatically configured!
✅ Authentication is enabled and cluster connections are encrypted.

ℹ️  Password for the elastic user (reset with `bin/elasticsearch-reset-password -u elastic`):
  password-generated-here

ℹ️  HTTP CA certificate SHA-256 fingerprint:
  440cc989e842409722cc8e2ee7b9443333aec95c71e6c2f0dc5d876dfc421e92

ℹ️  Configure Kibana to use this cluster:
• Run Kibana and click the configuration link in the terminal when Kibana starts.
• Copy the following enrollment token and paste it into Kibana in your browser (valid for the next 30 minutes):
  eyJ2ZXIiOiI4LjE0LjAiLCJhZHIiOlsiMTMwLjc5LjE2OS42OTo5MjAwIl0sImZnciI6IjQ0MGNjOTg5ZTg0MjQwOTcyMmNjOGUyZWU3Yjk0NDMzMzNhZWM5NWM3MWU2YzJmMGRjNWQ4NzZkZmM0MjFlOTIiLCJrZXkiOiJsN1VHd1pNQnFHY2N5QXFwUk9IczpnMkQ5THhBYlRZZWpPR1ZrcU9OMHZ3In0=

ℹ️  Configure other nodes to join this cluster:
• On this node:
  ⁃ Create an enrollment token with `bin/elasticsearch-create-enrollment-token -s node`.
  ⁃ Uncomment the transport.host setting at the end of config/elasticsearch.yml.
  ⁃ Restart Elasticsearch.
• On other nodes:
  ⁃ Start Elasticsearch with `bin/elasticsearch --enrollment-token <token>`, using the enrollment token that you generated.
```

### Problèmes connus

L'erreur suivante peut avoir lieu : "Flood stage disk watermark exceeded on all indices on this node will be marked read-only".

Voici une solution qui a fonctionné dans cette salle : https://stackoverflow.com/q/48119189

<a name="h2-installer-kibana"></a>
## Installer Kibana

Une fois dézippé (ce qui peut être assez lent dans la salle), faire

```
./bin/kibana
```

Une fenêtre de progression s'affiche pendant la procédure, à la fin de laquelle on verra un dialogue de connexion.

Saisir le token ou autres informations demandées selon les informations obtenues lors de l'installation d'ES. Le user est *elastic*.

La procédure peut être assez lente dans la salle (5 minutes la dernière fois testée).

<img src="./img/starting_elastic.png" alt="Démarrage ES" width="500"/>

<p></p>

<img src="./img/kibana_login.png" alt="Login Kibana" width="500"/>

Après démarrage, Kibana s'affiche sur http://localhost:5601

<a name="h3-variables-environnement"></a>

## Variables d'environnement

Nous allons utiliser ES avec `https`, avec une configuration de sécurité. Nous devons utiliser un mot de passe et un certificat pour les requêtes. Nous allons créer des variables d'environnement pour la chaîne de caractères correspondant au mot de passe et pour le chemin vers le certificat.

Pour créer ces variables, vous le ferez dans le fichier `~/.bash_profile` dans votre `$HOME`. Si le fichier n'existe pas déjà, vous pouvez le créer. Vous pouvez l'éditer avec `nano` ou `vim` en ligne de commande ou avec un éditeur graphique (il y a plusieurs dans la salle). Il est à vous de choisir les noms de variables, mais il faudra leur assigner la bonne valeur. P. ex. 
j
```bash
# dans ~/.bash_profile

export ESCERT=/chemin/vers/le/certificat
export ESPW=mot-de-passe-genere-lors-de-la-premiere-utilisation
```

Le chemin vers le certificat est `config/certs/http_ca.crt` depuis la racine de ES (dossier de ES). Donnez pourtant le chemin absolut `/adhome/XXXX/config/certs/http_ca.crt`, où `XXXX` est le reste du chemin qui est spécifique à chaque user.

Ensuite ces noms de variables d'environnement seront utilisés dans vos requêtes vers ES.

<a name="h1-premiers-pas"></a>
# Premiers pas

<a name="h1-comment-faire-requetes"></a>
## Comment faire les requêtes : Ligne de commande et console graphique de Kibana

Nous pouvons faire des requêtes vers ES en ligne de commande avec `curl`, ou avec la console graphique de Kibana. Dans ce tutoriel nous avons des exemples des deux types. 

Kibana a une console graphique qui permet de faire des requêtes et de voir les résultats en format graphique. Pour y accéder, il faut aller sur l'URL de chaque index, p. ex. http://localhost:5601/app/enterprise_search/content/search_indices/presse01 et cliquer sur la barre *Console*.

Cela affiche une console où on peut écrire des requêtes et voir les résultats en format graphique. La console garde également un historique des requêtes.

![Console Kibana](./img/kibana_search_console.png)

<a name="h1-verifier-serveur"></a>
## Vérifier si le serveur est accessible

```bash
curl --cacert $ESCERT -u elastic:$ESPW -XGET https://localhost:9200
```

La commmande fonctionne comme suit :

- `curl` permet de faire des requêtes HTTP(S) (semblable à l'utilité du module `requests` en Python)
- `--cacert` introduit le certificat pour les requêtes HTTPS
- `-u USER:PASSWORD` introduit le user et son mot de passe
- `-XGET` fait une requête `GET` (qui donne accès à une ressource existante) vers l'URL lui donné en argument

Réponse attendue est quelque chose comme le suivant (indique le nom du poste et la version d'ES entre autres informations) :

```json
{
  "name" : "nom-du-poste",
  "cluster_name" : "elasticsearch",
  "cluster_uuid" : "2qHx6avzQUO6eMfDRRWY3Q",
  "version" : {
    "number" : "8.17.0",
    "build_flavor" : "default",
    "build_type" : "tar",
    "build_hash" : "2b6a7fed44faa321997703718f07ee0420804b41",
    "build_date" : "2024-12-11T12:08:05.663969764Z",
    "build_snapshot" : false,
    "lucene_version" : "9.12.0",
    "minimum_wire_compatibility_version" : "7.17.0",
    "minimum_index_compatibility_version" : "7.0.0"
  },
  "tagline" : "You Know, for Search"
}
```

<a name="h1-creer-index"></a>
## Création d'un index

Maintenant il s'agit d'une requête `PUT` ; typiquement ce type de requête va créer une ressource (ici, un nouvel index, nommé `tutorial`).

```bash
curl --cacert $ESCERT -u elastic:$ESPW -XPUT https://localhost:9200/tutorial
```

Réponse attendue :

```json
{"acknowledged":true,"shards_acknowledged":true,"index":"tutorial"}
```

Nous pouvons aussi regarder dans Kibana, il faut cliquer dans l'onglet jaune *Elasticsearch* et ensuite dans le menu de gauche *Content / Indices*. Le nouvel index sera dans la liste (Ou aller sur l'URL http://localhost:5601/app/enterprise_search/content/search_indices).

![Création index kibana](./img/index_created_kibana.png)

<a name="h1-indexer-document"></a>
# Indexer un document

Il s'agit d'une requête `POST`, avec laquelle nous modifions une ressource (un index dans ce cas) existante.

```bash
curl -u elastic:$ESPW --cacert $ESCERT -XPOST -H "Content-Type: application/json" 'https://localhost:9200/tutorial/_doc/1' -d '{ "message": "Hello World!"}' 
```

Ça fonctionne comme suit :

- `-XPOST` demande à ce que la requête soit de type `POST`
- `-H` introduit les entêtes ou headers de la requête, ici un header concernant le type de contenu qui est envoyé au sesrveur par le biais de cette requête, avec `Content-Type` comme clé et `application/json` comme valeur.
- Dans l'URL, l'index à mettre à jour (`tutorial`) est mentionné, suivi de `/_doc/` et d'un identifiant pour le document à créer, ici `1`.
- Le document à indexer, introduit par l'argument `-d` est exprimé en JSON. Il ne contient qu'un seul champ `message` dans ce cas.

Vous pouvez également ajouter le paramètre `pretty` dans l'URL pour obtenir une réponse mieux formatée : `https://localhost:9200/tutorial/_doc/1?pretty`

Si l'index mentionné dans la requête n'existe pas, il sera créé. Pas besoin de créer un index auparavant. Mais cela peut être utile pour spécifier le type de champs de l'index avant d'y ajouter des documents (plus de détails sur ça plus tard).

En cas de succès, la réponse sera comme suit :

```json
{"_index":"tutorial","_id":"1","_version":1,"result":"created","_shards":{"total":2,"successful":1,"failed":0},"_seq_no":0,"_primary_term":2}
```

Le document sera visible sur Kibana, à l'URL http://localhost:5601/app/enterprise_search/content/search_indices/tutorial/documents


![document créé Kibana](./img/document_created_kibana.png)

<a name="h1-modifier-valeur-champ"></a>

# Modifier la valeur d'un champ pour un document déjà indexé

Peut se faire en changeant la valeur du champ dans la requête :

```bash
curl -u elastic:$ELASTICPW --cacert $ESCERT -XPOST -H "Content-Type:application/json" "https://localhost:9200/tutorial/_doc/1?pretty" -d '{ "message": "Hello, People from Earth!" }'
```

Réponse positive :

```json
{
  "_index" : "tutorial",
  "_id" : "1",
  "_version" : 2,
  "result" : "updated",
  "_shards" : {
    "total" : 2,
    "successful" : 1,
    "failed" : 0
  },
  "_seq_no" : 1,
  "_primary_term" : 2
}
```

La modification sera visible sur Kibana :

![Alt text](./img/kibana_atomic_doc_update.png)

<a name="h1-indexation-massive"></a>
## Indexation massive

Nous pouvons ajouter `_bulk` dams l'URL de la requête au lieu de `_doc`.

Ces mises à jour sont beaucoup plus rapides que document par document.

Nous pouvons voir le format de la requête (et des documents) dans l'exemple ci-dessous :

```bash
curl -u elastic:$ESPW --cacert $ESCERT -X POST "https://localhost:9200/_bulk?pretty" -H 'Content-Type: application/json' -d'
{ "index" : { "_index" : "tutorial" } }
{"name": "Revelation Space", "author": "Alastair Reynolds", "release_date": "2000-03-15", "page_count": 585}
{ "index" : { "_index" : "tutorial" } }
{"name": "1984", "author": "George Orwell", "release_date": "1985-06-01", "page_count": 328}
{ "index" : { "_index" : "tutorial" } }
{"name": "Fahrenheit 451", "author": "Ray Bradbury", "release_date": "1953-10-15", "page_count": 227}
{ "index" : { "_index" : "tutorial" } }
{"name": "Brave New World", "author": "Aldous Huxley", "release_date": "1932-06-01", "page_count": 268}
{ "index" : { "_index" : "tutorial" } }
{"name": "The Handmaids Tale", "author": "Margaret Atwood", "release_date": "1985-06-01", "page_count": 311}
{ "index" : { "_index" : "tutorial" } }
{"name": "Revelation Space Redux", "author": "Alastair Reynolds", "release_date": "2005-03-15", "page_count": 585}
{ "index" : { "_index" : "tutorial" } }
{"name": "1984 + 42", "author": "George Orwell", "release_date": "2024-06-01", "page_count": 328}
{ "index" : { "_index" : "tutorial" } }
{"name": "Fahrenheit 541", "author": "Ray Bradbury", "release_date": "1953-10-15", "page_count": 227}
{ "index" : { "_index" : "tutorial" } }
{"name": "Brave New World Yeah Right", "author": "Aldous Huxley", "release_date": "1992-06-01", "page_count": 268}
{ "index" : { "_index" : "tutorial" } }
{"name": "The Handmaids Tell Tale Heart", "author": "Margaret Atwood", "release_date": "1985-06-01", "page_count": 311}
'
```

Cela fonctionne comme suit :

- Chaque document est précédé de la syntaxe suivante : `{ "index" : { "_index" : "tutorial" } }`, où la valeur pour la clé ̀`_index` est le nom de l'index où se trouvent les documents. Le nom de l'index n'est donc pas mentionné dans l'URL, masi plutôt dans la ligne qui précède chaque document de la requête

En utilisant l'option `--data-binary`, il est possible d'indexer les documents stockés dans un fichier au lieu de les donne comme chaîne de caractères en ligne de commande, et dans ce cas le chemin vers le fichier est précédé de `@` :

```bash
curl -u elastic:$ESPW --cacert $ESCERT -H "Content-type: application/json" -X POST "https://localhost:9200/_bulk?pretty" --data-binary @documents_a_indexer
```

Une réponse positive sera semblable à celle déjà affichée ci-dessus pour la modification de la valeur d'un champ de l'index. Il y aura un élément dans la réponse pour chaque document indexé. Pour avoir une réponse moins verbeuse, on peut enlever le paramètre `pretty` (et `?` bien sûr).

Les documents indexés s'afficheront sur Kibana :

![Résultat bulk indexing Kibana](./img/bulk_indexing_kibana.png)

<a name="h1-effacer-documents"></a>
## Effacer des documents

### Avec `_delete_by_query`

La requête suivante effacerait tous les documents d'un index nommé `index_to_delete`

```bash
curl -u elastic:$ESPW --cacert $ESCERT -H "Content-type: application/json" -XPOST "https://localhost:9200/index_to_delete/_delete_by_query" -d '{"query": {"match_all": {}}}'
```

<a name="h1-mappings"></a>
# Les mappings : schéma de l'index

<a name="h2-les-mappings-et-y-acceder"></a>
## Les mappings et y accéder

En ES, le schéma (spécification des types de champ de l'index) est appelé *mapping*. 

En examinant le mapping existant, nous pouvons voir les types de champ actuels. Sur Kibana, on peut voir le mapping sur http://localhost:5601/app/enterprise_search/content/search_indices/NOM-DE-MON-INDEX/index_mappings. Cliquez sur JSON pour voir le schéma clairement en JSON. 

Nous pouvons aussi obtenir le schéma existant an ligne de commande :

```bash
curl -u elastic:$ESPW --cacert $ESCERT -XGET "https://localhost:9200/tutorial/_mapping?pretty"
```

Deux types de champ importants :
- [`text`](https://www.elastic.co/guide/en/elasticsearch/reference/current/text.html) pour la recherche plein texte (avec les prétraitements appropriés, comme la tokenisation, stemming, élimination des stopwords, etc.)
- [`keyword`](https://www.elastic.co/guide/en/elasticsearch/reference/current/keyword.html) pour la recherche exacte.

Lors de la première indexation, ES essaie de deviner le type de champ et configure le mapping en conséquence (comme Solr). Mais il est possible de modifier le mapping et ré-indexer après effacer les documents (comme avec Solr). 

<a name="h2-modification-mappings"></a>
## Modification des mappings

Modifier le mapping a le même type d'utilité que mettre à jour le fichier `managed-schema` dans Solr. P. ex. nous pouvons définir un champ comme contenant du texte en français ou dans une autre langue.

Pour modifier les mappings, nous spécifions les caractéristiques souhaitées dans le schéma dans un document (dans la requête ci-dessous, il s'appelle `mapping_french_00.json`), et nous mettons le schéma à jour avec une requête `PUT` :

```bash
curl -u elastic:$ESPW --cacert $ESCERT -X PUT "https://localhost:9200/tutorial/_mapping?pretty" -H 'Content-Type: application/json' --data-binary @queries/mapping_french_01.json
```

Le contenu du document qui spécifie les mappings est décrit dans la section suivante.

<a name="h2-exemple-mappings"></a>
## Exemple

Nous allons indexer les documents dans `documents/all_out_00.json` dans un index  nommé `presse00`.

```bash
curl -u elastic:$ESPW --cacert $ESCERT -H "Content-type: application/json" -X POST "https://localhost:9200/presse00/_bulk?pretty" --data-binary @documents/all_out_00
```

En cherchant le mot *cheval*, nous voyons qu'il y a 14 documents qui correspondent à la requête.

![cheval_14.png](img/cheval_14.png)

Ensuite nous créons un deuxième index `presse01` et modifions le mapping pour spécifier que le champ `description` contient du texte en français.

**Création de l'index**

```bash
curl --cacert $ESCERT -u elastic:$ESPW -XPUT https://localhost:9200/presse01
```

**Modification du schéma**

```bash
curl -u elastic:$ESPW --cacert $ESCERT -X PUT "https://localhost:9200/presse01/_mapping?pretty" -H 'Content-Type: application/json' --data-binary @queries/mapping_french_01.json
```

Le contenu du fichier décrivant les nouveaux mappings est comme suit, et après exécuter la commande, on pourra voir ces mappings sur l'index (avec Kibana) :

```json
{
    "properties": {
      "description": {
        "type": "text",
        "analyzer": "french",
        "fields": {
          "keyword": {
            "type": "keyword",
            "ignore_above": 256
          }
        }
      },
      "title": {
        "type": "text",
        "analyzer": "french",
        "fields": {
          "keyword": {
            "type": "keyword",
            "ignore_above": 256
          }
        }
      }
    }
}
```

**Indexation**

```bash
curl -u elastic:$ESPW --cacert $ESCERT -H "Content-type: application/json" -X POST "https://localhost:9200/presse00/_bulk?pretty" --data-binary @documents/all_out_01
```

Avec le schéma qui spécifie des champs en français, en cherchant le mot *cheval*, nous voyons qu'il y a 24 documents correspondant à la requête.

![cheval_24.png](img/cheval_24.png)

<a name="h1-recherches"></a>
# Recherches

<a name="h2-recherches-texte-integral-basiques"></a>
## Recherches en texte intégral de base

En ligne de commande :

```bash
curl -u elastic:$ESPW --cacert $ESCERT -XGET "https://localhost:9200/presse01/_search?q=victime&pretty"
```

Dans la console de Kibana on peut faire plus simple pour le même résultat :

```
GET /presse01/_search?q=victime
```

Dans le résultat, un champ `hits` donne le nombre de documents correspondant à la requête, et un champ `hits` à son intérieur donne les documents eux-mêmes.

```json
{
  "took" : 2,
  "timed_out" : false,
  "_shards" : {
    "total" : 1,
    "successful" : 1,
    "skipped" : 0,
    "failed" : 0
  },
  "hits" : {
    "total" : {
      "value" : 123,
      "relation" : "eq"
    },
    "max_score" : 8.290699,
    "hits": [ ... ]
  }
}

```

Pour chercher le mot dans un champ concret (ici, `title`) :

```bash
# ligne de commande
curl -u elastic:$ESPW --cacert $ESCERT -XGET "https://localhost:9200/presse01/_search?q=title:victime&pretty"
```

```
# console Kibana
GET /presse01/_search?q=title:victime
```

Choisir les champs de la sortie à afficher (ici, `title` et `pubname`) :

```bash
curl -u elastic:$ESPW --cacert $ESCERT -XGET "https://localhost:9200/presse01/_search?q=title:victime&pretty&_source=title,pubname"
```

Spécifier le nombre de documents à afficher avec `size` (ici, 1) ; c'est léquivalent de `rows` en Solr :

```bash
curl -u elastic:$ESPW --cacert $ESCERT -XGET "https://localhost:9200/presse01/_search?q=title:victime&size=1&pretty"
```

Spécifier à partir de quel document afficher (ici, à partir du 3ème) ; c'est l'équivalent de `start` en Solr :

```bash
curl -u elastic:$ESPW --cacert $ESCERT -XGET "https://localhost:9200/presse01/_search?q=title:victime&size=10&from=2&pretty"
```

Retourner tous les documents de l'index :

```bash
curl -u elastic:$ESPW --cacert $ESCERT -XGET "https://localhost:9200/presse01/_search?q=*&pretty"
```

<a name="h2-recherches-basiques-en-json"></a>

## Recherches de base avec des requêtes JSON

Les requêtes JSON sont plus flexibles que les requêtes en ligne de commande. Elles permettent de spécifier plus de paramètres, et de faire des requêtes plus complexes.

<a name="h3-requetes-match"></a>
### Requêtes de type `match`

Dans l'exemple suivant, on cherche le mot *victime* dans le champ `title` :

**Avec la console de Kibana :**

```json
GET /presse01/_search
{
  "query": {
    "match": {
      "title": "victime"
    }
  }
}
```

Pour spécifier d'autres paramètres (comme le nombre de documents à afficher, ou à partir de quel document afficher), on fait comme suit :

```json
GET /presse01/_search
{
  "query": {
    "match": {
      "title": "victime"
    }
  },
  "size": 1,
  "from": 2
}
```

L'exemple affiche le 3ème document correspondant à la requête.

**En ligne de commande :**

Nous écrivons la requête en JSON dans un fichier, et on l'envoie avec `--data-binary` :

```bash
curl -u elastic:$ESPW --cacert $ESCERT -H "Content-type: application/json" -XGET "https://localhost:9200/presse01/_search?pretty" --data-binary @queries/query_match_01.json
```

Contenu du fichier : 

```json
{
  "query": {
    "match": {
      "title": "victime"
    }
  },
  "_source": ["title"],
  "size": 1,
  "from": 2
}
```

<a name="h4-changer-operateur-recherche></a>

#### Changer l'opérateur de recherche

En Lucène, l'opérateur par défaut est OR. Pour changer à AND, on peut ajouter un paramètre `operator` dans la requêt, comme dans la requête suivante qui cherche les documents où des correspondances pour les mots *victime* et *racisme* sont trouvées dans le champ `title` :

```json
GET /presse01/_search
{
  "query": {
    "match": {
      "title": {
        "query": "victime racisme",
        "operator": "and"
      }
    }
  }
}
```

<a name="h4-nombre-minimal-corresp"></a>

#### Demander un nombre minimal de termes correspondants

La requête suivante cherche dans le champ `title` les documents qui contiennent au moins deux des termes *violence*, *politique*, et *mineur*, en n'affichant que le champ `title` :

À noter que dans les requêtes JSON, les champs à afficher sont spécifiés dans `_source` mais à l'extérieur de `query`. : 

```json
GET /presse01/_search
{
  "_source": ["title"],
  "query": {
    "match": {
      "title": {
        "query": "violence politique mineur",
        "minimum_should_match": 2 }
    }
  }
 
}
```

<a name="h4-multi-match"></a>

#### Choisir les champs où chercher avec `multi_match`

On peut les lister dans un champ `fields`, et il est possible de spécifier un boost pour chaque champ, p. ex. ici `^5`pour le champ `title` (cette syntaxe est aussi utilisée dans Solr) :

```json
GET /presse01/_search
{
"_source": ["title"],
  "query": {
    "multi_match": {
      "query": "violence mineurs",
      "fields": ["title^5", "description^2"] 
    }
  }
}
```

<a name="h3-recherches-exactes-et-filtrage"></a>

### Requêtes de type `term` : Recherche exacte et filtrage

Dans l'exemple, on cherche les documents qui contiennent le mot *migrants*, et on filtre pour retenir ceux qui ont la chaîne *AFP Infos Mondiales*, en recherche exact, dans le champ `pubname.keyword`. Ce champ est une copie du contenu `pubname` du document, mais sans pré-traitements. Dans le schéma, on a spécifié que le champ `pubname` est de type `text` et le champ `pubname.keyword` est de type `keyword` :

```json
      "pubname": {
        "type": "text",
        "fields": {
          "keyword": {
            "type": "keyword",
            "ignore_above": 256
          }
        }
      },
```

Nous écrivons la requête dans un document (ici `facet_query_01.json`) et l'envoyons par requête `GET` :

```bash
curl -u elastic:$ESPW --cacert $ESCERT -H "Content-type: application/json" -XGET "https://localhost:9200/tutorial/_search?pretty" --data-binary facet_query_01.json
```

Le contenu du fichier est  le suivant :

```json
{
  "_source": ["title", "pubname"],
  "query": {
    "bool": {
      "must": [
        { "match":
          { "description": "migrants" }
        }
      ],
      "filter": [
        { "term": { "pubname.keyword": "AFP Infos Mondiales" } }
      ]
    }
  }
}
```

Il est possible de filtrer sur un champ qui n'est pas défini pour la recherche exacte  (mais il pourrait être plus lent que les filtres exacts). Dans l'exemple suivant, la requête `term` est faite sur le champ `pubname`, qui est de type `text` :

```json
POST /presse01/_search
{
  "_source": ["title", "pubname"],
  "query": {
    "bool": {
      "must": [
        { "match":
          { "description": "migrants" }
        }
      ],
      "filter": [
        { "match": { "pubname": "afp" } }
      ]
    }
  }
}
```

<a name="h1-facettes"></a>
## Facettes

<a name="h2-affichage-de-facettes"></a>

### Affichage de facettes

Comment obtenir des facettes, ou compte des valeurs dans un champ (la facette) des documents correspondant à la requête ?

Il faut ajouter une clé `aggs` dans la requête. Le champ à mentionner est la version `keyword` du champ pour lequel on veut obtenir des facettes, ici `pubname.keyword`. 

```json
{
  "query": {
    "bool": {
      "must": [
        { "match": { "description": "migrants" } }
      ],
      "filter": [
        { "term": { "pubname": "afp" } }
      ]
    }
  },
  "aggs": {
    "pubname_str": {
      "terms": { "field": "pubname.keyword" }
    }
  }
}
```

Dans la réponse, un champ `aggregations` donnera le compte de chaque terme dans le champ `pubname.keyword` trouvé dans les documents retournés pour la requête.

```json
"aggregations": {
    "pubname_str": {
      "doc_count_error_upper_bound": 0,
      "sum_other_doc_count": 0,
      "buckets": [
        {
          "key": "AFP Infos Françaises",
          "doc_count": 3
        },
        {
          "key": "AFP Infos Mondiales",
          "doc_count": 2
        },
        {
          "key": "AFP - Journal Internet AFP (français)",
          "doc_count": 1
        }
      ]
    }
  }
```

<a name="h1-recherche-semantique"></a>
# Recherche sémantique

ES peut communiquer avec l'API de plusieurs plateformes de modèles de langue, incluant des LLM, pour obtenir des embeddings. Il est possible d'indexer ces embeddings dans ES et de faire des recherches par embedding ; ES obtient également l'embedding pour la requête et le compare avec les embeddings des documents pour trouver les plus similaires.

Dans l'exemple nous demandons à ES de créer des embeddings avec Mistral, profitant de la clé gratuite en ce moment (décembre 2024).

Plus d'information sur cette fonctionnalité s'affiche sur Kibana : http://localhost:5601/app/enterprise_search/semantic_search?model_example=elser

<a name="h2-creation-point-acces"></a>
## Création d'un point d'accès dans ES qui communique avec l'API d'un modèle de langue

Depuis la console Kibana :

```json
PUT _inference/text_embedding/mistral-embeddings-test-001
{
  "service": "mistral",
  "service_settings": {
    "api_key": "votre-cle-api-pour-mistral",
    "model": "mistral-embed" 
  }
}
``` 

Une réponse positive donnera les informations suivantes :

```json
{
  "inference_id": "mistral-embeddings-test-001",
  "task_type": "text_embedding",
  "service": "mistral",
  "service_settings": {
    "model": "mistral-embed",
    "dimensions": 1024,
    "similarity": "dot_product",
    "rate_limit": {
      "requests_per_minute": 240
    }
  },
  "task_settings": {}
}
```

<a name="h2-preparation-index-recherche-semantique"></a>

## Préparation de l'index pour stocker les embeddings

Nous allons maintenant créer un index et spécifier dans le schéma que les champs `title` et `description` vont stocker des embeddings du type nommé `semantic_text` dans ES, compatible avec plusieur types d'embeddings (voir la liste [ici](https://www.elastic.co/guide/en/elasticsearch/reference/8.15/put-inference-api.html)).


Création de l'index en ligne de commande (c'est également possible avec la console Kibana) :

```bash
curl --cacert $ESCERT -u elastic:$ESPW -XPUT https://localhost:9200/presse04
```

Modification du mapping (avec la console Kibana, mais c'est possible en ligne de commande) :

```json

PUT /presse04/_mapping
{
    "properties": {
      "description": {
        "type": "semantic_text",
        "inference_id": "mistral-embeddings-test-001"
      },
      "title": {
        "type": "semantic_text",
        "inference_id": "mistral-embeddings-test-001"
      }

    }
}
```

<a name="h2-indexation-recherche-semantique"></a>
## Indexation

Nous indexons 25 documents pour aller rapidement, disponibles dans `documents/all_out_04`. Il serait possible d'indexer le corpus complet de 3 050 documents, mais plus lent selon le temps demandé pour obtenir les embeddings avec Mistral.

```bash
curl -u elastic:$ESPW --cacert $ESCERT -H "Content-type: application/json" -X POST "https://localhost:9200/_bulk?pretty" --data-binary @documents/all_out_04
```

<a name="h2-recherche-recherche-semantique"></a>
## Recherche

Nous cherchons la requête *énergies renouvellables* dans le champs `description` :

```json
POST /presse04/_search
{
  "size" : 2,
  "query" : {
    "semantic": {
      "field": "description", 
      "query": "énergies renouvellables" 
    }
  }
}
```

Le premier document retourné commence avec « Jean-Pierre Laurent est un pionnier de l'éolien français », ce qui semble être un bon résultat.

<a name="h1-visualisations-kibana"></a>
# Les dashboards dans Kibana

Avant de créer un dashboard, il faut créer une *Data view*. Cela se fait avec le menu de gauche, dans *Analytics / Discover*.

![Création index pattern](./img/creation_index_pattern.png)

Ensuite, on peut créer un dashboard avec *Analytics / Dashboards*.

Nous pouvons choisir un des index pattern et ajouter des visualisations. 

Au début, le dashboard est vide, mais on peut ajouter des visualisations par glisser-déposer des champs.

![img.png](img/dash_vide.png)

Ici un treemap pour les noms des publicationa a été créé. On voit qu'il est possible de configurer le diagramme avec le menu de droite.

![img.png](img/treemap_pubnames.png)

Le dashboard suivant a été obtenu avec l'index `tutorial` qui contient quelques données sur des livres et leurs auteurs ou autrices.

![dash_authors.png](./img/dash_authors.png)

Le diagramme circulaire a été obtenu pour le champ `author.keyword` et le diagramme à barres pour le champ `release_date` et `author_keyword`. La hauteur des barres est déterminée par `release_date` et la couleur par `author.keyword`. 
