# TP - Recherche d'information partie 1 : Recherche lexicale avec Solr

1. [Introduction](#intro)
1. [Téléchargement et démarrage de Solr](#teledem)
    1. [Téléchargement](#tele)
    1. [Démarrage](#dem)
1. [Indexation de documents](#indexationEtc)
    1. [Création du core](#creatcore)
    1. [Structure du core](#structureCore)
    1. [Indexation](#indexation)
    1. [Validation](#valid)
1. [Configuration du schéma : `managed-schema`](#configSchema)
    1. [Champs créés dynamiquement par Solr](#champs-dyn-solr)
    1. [Changement des options linguistiques de ces champs pour le français](#champs_fr)
    1. [Autres informations sur les champs: recherche exacte vs. inexacte](#champ_vs_champ_str)
    1. [Recharger la configuration après modification](#reloadConfig)
1. [Effacer les contenus de l'index et réindexer](#effacerReindexer)
1. [Faire une requête simple](#requeteSimple)
1. [Gestion de la recherche : `solrconfig.xml`](#gestionRecherche)
1. [Interface de navigation de résultats](#interface)
    1. [Utilité](#interfaceUtilite)
    1. [Configuration](#configuration)
    1. [Templates Velocity](#templatesVelocity)


## Introduction <a name="intro"/>

Cette partie du TP vise à pratiquer les activités suivantes :
  - Indexation dans Solr d'une collection de documents déjà préparée en format XML prêt à indexer
      - Les documents ont été annotés avec des entités nommées, qui seront utilisées comme facettes pour filtrer les résultats de recherche
  - Opérations de base sur l'index : modifier la configuration, recharger après modifications, effacer des documents, réindexer des documents
  - Adaptation du schéma Solr (fichier `managed-schema`) à l'indexation de textes en français
  - Modification de la gestion de la recherche dans le fichier `solrconfig.xml` : champ de recherche par défaut, facettes à calculer, poids des champs
  - Modification des templates d'affichage HTML par défaut, afin d'afficher des entités nommées comme facette

Nous travaillons avec Solr **<html><span style="color:#e63647">8.3.0</span></html>**, qui date d'octobre 2019. À télécharger [ici](https://archive.apache.org/dist/lucene/solr/8.3.0/solr-8.3.0.zip)

Les diaporamas sur moodle donnent plus de contexte. Bien que l'énoncé du TP détaille toutes les étapes à suivre, si vous avez besoin de plus d'information sur le moteur Solr, le diaporama *03 - Introduction à la plate-forme de recherche Solr* sur moodle donne quelques renseignements additionnels

Si besoin, il est également possible de consulter une **vidéo** disponible sur moodle qui montre les étapes du TP

## Téléchargement et démarrage de Solr  <a name="teledem"/>

## Téléchargement  <a name="tele"/>

```shell
# On assume notre home comme dossier de travail
cd ~
wget "https://archive.apache.org/dist/lucene/solr/8.3.0/solr-8.3.0.zip"
unzip solr-8.3.0.zip -d .
# lien symbolique pour accès facile
ln -s solr-8.3.0 solr
cd solr 
```

Une fois l'outil dézippé, nous aurons l'arborescence suivante:

<img src="images/arborescence_outil_solr.png" alt="arborescence de l'outil Solr" width="250"/>

## Démarrage  <a name="dem"/>

```shell
# depuis la racine de l'outil
./bin/solr start
```

Par défaut, Solr écoute sur le port 8983. Le port peut être changé au démarrage avec l'option `-p`, (p. ex. `./bin/solr start -p 8984`)

La sortie de la commande `start` est la suivante (après d'éventuels warnings selon les caractéristiques du système). Le message *Started Solr server on port NNNN* indique un démarrage avec succès.

```
Waiting up to 180 seconds to see Solr running on port 8983 [|]  
Started Solr server on port 8983 (pid=7380). Happy searching!
```

Une fois l'outil démarré, une interface graphique d'administration sera disponible sur http://localhost:8983/solr

Une capture d'écran de la console d'administration. On voit qu'aucun index n'a été créé grâce au message *No cores available* à gauche au milieu. En Solr, un *core* veut dire un index (données indexées) + ses fichiers de configuration

![interface graphique d'administration Solr](images/console-admin-solr.png "Console admin Solr")

## Indexation de documents <a name="indexationEtc">

### Création du core <a name="creatcore">

La commande `create` s'en occupe. Le paramètre `-c` (obligatoire) donne le nom du core

Nous allons créer un core nommé `presse` (ce nom d'index sera utilisé dans toutes les commandes de ce TP): 

`./bin/solr create -c presse`

Le message *Created new core 'presse'* confirmera le succès. Il peut y avoir des warnings avant, qui ne devraient pas avoir d'impact pour ce TP

Le core créé sera visible dans la console d'admin, on peut y accéder avec le menu dépliant *Core selector* à gauche.

On voit que l'index est vide (*Num Docs: 0*)

![core selector](images/core-selector.png "Sélection du core 'presse'")

### Structure du core <a name="structureCore"/>

Une fois le core créé, il aura la structure de dossiers affichée dans l'image suivante. Un core inclut l'index, les données indexées en format binarisé, et plusieurs fichiers de configuration
- Le dossier `conf` contient la configuration du core
- Le dossier `data` contient les données indexées (à ce stade il n'y a pas encore de données)

Les fichiers de configuration que nous allons manipuler dans le TP sont marqués en rose.

<img src="images/core_solr_dir_just_created_annot.png" alt="arborescence d'un core vide" width="250"/>


### Indexation <a name="indexation">


Nous allons utiliser les documents sur GitLab, fichier  nommé [(europresse_xml_2_ents.zip)](./01_recherche_lexicale/europresse_xml_2_ents.zip)

Le format du document est le suivant (voir exemple ci-dessous):
  - La balise initiale `<add>` est la commande pour Solr : ajouter à l'index
  - La balise `<doc>` délimite chaque document (on pourrait ajouter à l'index plusieurs documents dans la même instruction `add`)
  - Les champs sous `doc` seront indexés:
    - Le champ `id` (`<field name="id" ...`), qui est généralement obligatoire dans les configurations Solr, contient un ID unique pour chaque document
    - Le champ `title` est souvent obligatoire, c'est le titre du document
    - Le champ `description` comporte le gros du contenu (il s'agit d'un corpus de nouvelles de presse)
    - `pubname` contient le nom de la publication.
    - `pdate` est la date de publication
    - Tous les champs commençant par `enti_` : C'est des entités nommées (si vous voulez plus d'informations sur le concept d'entité nommée, voir diaporama *Enrichissement de la recherche d'information par le TAL*, sur moodle).


```xml
<?xml version='1.0' encoding='utf8'?>
<add>
  <doc>
    <field name="title">Wauquiez: l'annonce de Collomb, "surréaliste", montre "l'affaiblissement" de Macron</field>
    <field name="id">eupr_000002.xml</field>
    <field name="description"><![CDATA[
Paris (AFP) - L'annonce "surréaliste" de Gérard Collomb, qui a dit son intention de quitter le gouvernement en 2019 pour mener campagne municipale à Lyon, démontre "l'affaiblissement" d'Emmanuel Macron, a jugé mardi Laurent Wauquiez, président des Républicains (LR) et de la région Auvergne-Rhône-Alpes.
"Penser qu'on puisse avoir un ministre de l'Intérieur à mi-temps montre bien la déconnexion du pouvoir sur l'insécurité subie par les Français", a déclaré M. Wauquiez lors de la réunion du groupe LR à l'Assemblée, selon des participants.
]]></field>
    <field name="url">http://localhost:8983/solr/eufr/browse?q=id:eupr_000002.xml</field>
    <field name="pdate">2018-09-18T00:00:00Z</field>
    <field name="pubname">AFP Doc</field>
    <field name="section">Inconnu</field>
    <field name="enti_date">mai</field>
    <field name="enti_person">Gérard Collomb</field>
    <field name="enti_person">Laurent Wauquiez</field>
    <field name="enti_date">2019</field>
    <field name="enti_date">mardi</field>
    <field name="enti_person">Emmanuel Macron</field></doc>
</add>
```

Pour indexer, dézipper `europresse_xml_2_ents.zip` et aller à l'intérieur du dossier.

Depuis le dossier contenant les documents, on pourra indexer avec `curl` et la commande `update` de l'API HTTP de Solr

```shell
# indexer
for x in $(ls) ; do curl -F "fileupload=@$x" "http://localhost:8983/solr/presse/update" ; done
```

### Validation <a name="valid">

Après modifier le contenu de l'index (ajout ou suppression de documents), il faudra valider la modification par une instruction `commit`, également possible avec `curl` ou autre programme permettant une requête `GET`

```shell
# valider
curl "http://localhost:8983/solr/presse/update?commit=true"
```

Pour chaque document pour lequel l'indexation réussit, le message de succès sera du type suivant (avec *status 0*)

```xml
<lst name="responseHeader">
  <int name="status">0</int>
  <int name="QTime">0</int>
</lst>
```

Le message de succès pour le commit porte aussi la valeur *status 0*

```json
{"responseHeader":{
 "status":0,
 "QTime":9}}
```

Une fois les ajouts validés, on verra le nombre de documents indexés dans la console d'admin (*Num Docs: 3038*)

<img src="images/docs-valides.png" alt="Index après validation des documents ajoutés" width="500"/>

### Configuration du schéma : `managed-schema` <a name="configSchema">

L'emplacement du schéma Solr est `server/solr/presse/conf/managed-schema` depuis la racine de l'instance Solr.

#### Champs créés dynamiquement par Solr <a name="champs-dyn-solr"/>

Lors de l'indéxation, Solr a dynamiquement ajouté au schéma des champs selon les champs qui sont trouvés dans les documents d'entrée.

Nous pouvons nous attendre à trouver dans le schéma des champs comme `description`, `pubname`, `enti_person` et autres, cars ils étaient présents dans les documents a indexer. Solr infère le type pour le champ automatiquement.

Or, le schéma de base généré automatiquement par Solr peut être amélioré : Nous changerons la langue des champs textuels (par défaut en anglais) vers le français, comme décrit ci-dessous.

Voici les champs ajoutés par Solr au schéma (nous verrons un peu plus bas comment changer la langue pour ces champs):

```xml
  <field name="description" type="text_general"/>
  <field name="enti_date" type="text_general"/>
  <field name="enti_location" type="text_general"/>
  <field name="enti_money" type="text_general"/>
  <field name="enti_organization" type="text_general"/>
  <field name="enti_person" type="text_general"/>
  <field name="enti_time" type="text_general"/>
  <field name="id" type="string" multiValued="false" indexed="true" required="true" stored="true"/>
  <field name="pdate" type="pdates"/>
  <field name="pubname" type="text_general"/>
  <field name="section" type="text_general"/>
  <field name="title" type="text_general"/>
  <field name="url" type="text_general"/>
```

En plus de ces champs, vers la fin du schema nous verrons qu'il y a plusieurs champs de type `copyField` qui impliquent les champs ajoutés au schéma par Solr. Les champs source sont les mêmes déja vus ci-dessus, les champs dest (destination) portent le même nom, mais en ajoutant *_str* à celui-ci :

```xml
  <copyField source="pubname" dest="pubname_str" maxChars="256"/>
  <copyField source="enti_date" dest="enti_date_str" maxChars="256"/>
  <copyField source="enti_money" dest="enti_money_str" maxChars="256"/>
  <copyField source="description" dest="description_str" maxChars="256"/>
  <copyField source="section" dest="section_str" maxChars="256"/>
  <copyField source="title" dest="title_str" maxChars="256"/>
  <copyField source="enti_person" dest="enti_person_str" maxChars="256"/>
  <copyField source="enti_location" dest="enti_location_str" maxChars="256"/>
  <copyField source="url" dest="url_str" maxChars="256"/>
  <copyField source="enti_organization" dest="enti_organization_str" maxChars="256"/>
  <copyField source="enti_time" dest="enti_time_str" maxChars="256"/>
```

Les champs dont le nom finit par *_str* sont des champs de recherche *exacte*, sans application de la tokénisation ou de la racinisation (stemming).

Solr a donc indexé les champs des documents d'origine de deux façons différentes :
- En appliquant un pré-traitement (tokénisation, stemming), par défaut pour l'anglais (champs de type `text_general`, noms des champs comme dans les documents du corpus)
- Sans appliquer de pré-traitement, laissant le contenu original tel quel (pour recherche exacte, type `strings`, noms des champs ajoutent `_str` au nom dans les documents d'origine)

#### Changement des options linguistiques de ces champs pour le français <a name="champs_fr"/>

Nous allons changer les options linguistiques des champs pertinents, afin que la tokénisation et racinisation (stemming) françaises soient appliquées, en faisant comme ci-dessous.
- Les champs de texte courant (titre, contenu de l'article etc.), ainsi que les champs pour les entités, qui commencent par *enti_* passeront au type `text_fr`

```xml
  <field name="description" type="text_fr"/>
  <field name="pubname" type="text_fr"/>
  <field name="section" type="text_fr"/>
  <field name="title" type="text_fr"/>
```

Dans le cas des champs pour les entités, il peut y avoir plus d'une valeur par champ (plus d'un élément `enti_person`, `enti_location` etc par document). Il conviendra d'ajouter l'attribut `multiValued="true"`.

```xml
  <field name="enti_date" type="text_fr" multiValued="true"/>
  <field name="enti_location" type="text_fr" multiValued="true"/>
  <field name="enti_money" type="text_fr" multiValued="true"/>
  <field name="enti_organization" type="text_fr" multiValued="true"/>
  <field name="enti_person" type="text_fr" multiValued="true"/>
  <field name="enti_time" type="text_fr" multiValued="true"/>
```

#### Autres informations sur les champs: recherche exacte vs. inexacte <a name="champ_vs_champ_str"/>

Nous exploiterons les champs `enti_ ... _str`, en recherche exacte avec le type `strings`, pour afficher les entités dans l'interface, afin qu'elles soient montrés comme des facettes cliquables pour filtrer les résultats. Autrement, on pourrait avoir un mauvais affichage, car l'entité tokénisée et racinisée s'afficherait, au lieu de l'entité exacte, p.ex. *États-Unis* pourrait s'afficher comme *État* et ensuite dans une facette séparée *Uni*. 

Au même temps, nous voulons pouvour faire des recherches inexactes dans les entités : Nous pourrions avoir *Emmanuel Macron* comme facette mais nous voulons que le terme soit trouvé quand on cherche *Macron* dans les entités. Les champs `enti_` de type `text_fr` permettront cette utilisation. 

Note : Le type `strings` est déjà `multiValued` par défaut, donc il n'était pas nécessaire d'ajouter l'attribut, contrairement aux champs en type `text_fr`, où il .

#### Recharger la configuration après modification <a name="reloadConfig"/>

**<html><span style="color:#e63647;font-weight:bold">Après effectuer les modifications décrites dans </span><span style="color:#e63647;font-weight:bold;font-family:monospace">managed-schema</span><span style="color:#e63647;font-weight:bold">, il faut reloader le core, avec la requête </span><span style="color:#e63647;font-weight:bold;font-family:monospace">GET</span><span style="color:#e63647;font-weight:bold"> suivante:</span></html>**


```shell
curl "http://localhost:8983/solr/admin/cores?action=RELOAD&core=presse"
```

### Effacer les contenus de l'index et réindexer <a name="effacerReindexer">
Si nous avons modifiés les propriétés des champs de l'index, il est nécessaire de réindexer, afin que les nouvelles options d'analyse des champs soient prises en compte.

Nous allons effacer les contenus et réindexer.

Pour effacer les contenus:

- Activer l'option qui permettra de faire des requêtes `GET` pour effacer le contenu (il faut l'activer une fois par core):

```shell
curl -H 'Content-type:application/json' -d '{"set-property": {"requestDispatcher.requestParsers.enableRemoteStreaming": true}, "set-property": {"requestDispatcher.requestParsers.enableStreamBody": true}}' http://localhost:8983/api/cores/presse/config
```

- Effacer (il faut valider la modification par `commit` après):

```shell
curl "http://localhost:8983/solr/presse/update?stream.body=%3Cdelete%3E%3Cquery%3E*:*%3C/query%3E%3C/delete%3E"
curl "http://localhost:8983/solr/presse/update?commit=true"
```

Une fois la modification validée, on verra *NumDocs: 0* à nouveau dans la console d'admin

Pour réindexer, faire comme déjà vu avant, depuis le dossier contenant les documents à indexer:

```shell
# indexer
for x in $(ls) ; do curl -F "fileupload=@$x" "http://localhost:8983/solr/presse/update" ; done

# valider
curl "http://localhost:8983/solr/presse/update?commit=true"
```


## Faire une requête simple <a name="requeteSimple"/>

Maintenant l'index est prêt à accueillir des documents en français ; le stemming pour le français s'applique.

Nous pouvons faire une requête qui montre que la configuration s'applique. On peut coller la requête `GET` suivante dans le navigateur :

```
http://localhost:8983/solr/presse/select?q=description%3Acheval
```

La syntaxe de la requête est comme suit :

- `select` est un gestionnaire de requête (RequestHandler) qui retourne du JSON par défaut
- `q` est le paramètre qui introduit la requête
- `description:cheval` (url-encodé ci-dessus) demande de chercher l'expression `cheval` dans le champ `description` ; ce champ correspond au corps des articles de presse indexés

Une description plus complète de la syntaxe des requêtes Solr était fournie dans le diaporama, disponible sur moodle, *03 - Introduction à la plate-forme de recherche Solr*, diapos 43-45.

Dans la réponse, il y a d'abord une entête, et, après, la réponse en tant que telle. Elle donne le nombre de documents trouvés (`numFound: 24`), la position dans la liste de documents trouvés où nous sommes (`start: 0`), et, ensuite, chaque document qui correspond à la requête.

Dans un navigateur web, le début de la réponse se voit comme dans l'image ci-dessous. (Si nous faisons la requête avec un autre outil permettant des requêtes `GET` comme `curl`on n'aura pas le formatage, mais les données retournées par Solr seront les mêmes).

<img src="images/debut-response-json.png" alt="début response json" width="800"/>

En appelant le gestionnaire de requête `browse` plutôt que `select` depuis un navigateur, nous obtiendrons un affichage en HTML.

<img src="images/cheval-browse.png" alt="début response json" width="800"/>

## Gestion de la recherche : `solrconfig.xml` <a name="gestionRecherche"/>

La configuration ci-dessus peut être améliorée dans plusieurs aspects:
- Dans l'interface graphique, pour la requête précédente, il nous a fallu spécifier le champ (`description`) où le terme devait être cherché, ce qui n'est pas pratique. Il est plus pratique de définir un **champ de recherche par défaut** et de ne spécifier les champs qu'en cas d'exception.
- Dans l'affichage HTML, nous ne voyons pas les **facettes**, alors qu'elles seraient intéressantes pour filtrer les résultats
- Dans l'affichage HTML, le terme recherché n'est pas mis en **surbrillance** dans les résultats de recherche, alors que ce serait plus convivial pour les utilisateurs.trices

Tous ces éléments (le champ de recherche par défaut, l'ensemble de champs à retourner dans la réponse, les facettes à afficher, et la surbrillance des résultats), peuvent être gérés dans le fichier `solrconfig.xml`.

Dans ce fichier, plusieurs éléments `RequestHandler` ("gestionnaire de recherche") sont définis. Ils spécifient des options dans le sens évoqué.

Par défaut en Solr 8.3.0, qui est la version utilisée dans ce TP, le requestHandler défini pour l'affichage HTML est nommé `browse`, mais on peut améliorer sa définition.

Un `solrconfig.xml` avec les options déjà modifiées sur `browse` se trouve sur [GitLab](01_recherche_lexicale/config_solr/mdtal/solrconfig.xml)

La configuration est reproduite ci-dessous. Les commentaires XML décrivent le rôle de chaque élément de la configuration. Ils étaient par ailleurs expliqués dans le diaporama, disponible sur moodle, *03 - Introduction à la plate-forme de recherche Solr*, diapos 46-49.
 

```xml
<requestHandler name="/browse" class="solr.SearchHandler" useParams="query,facets,velocity,browse">
    <lst name="defaults">
      <str name="echoParams">all</str>

       <!-- champ de recherche par défaut -->
       <str name="df">description</str>

       <!-- Facettes qui seront calculées (disponibles donc pour affichage et filtrage des résultats) -->
       <str name="facet">on</str>
       <str name="facet.field">enti_location_str</str>
       <str name="facet.field">enti_organization_str</str>
       <str name="facet.field">pdate</str>
       <str name="facet.field">pubname_str</str>
       <!-- nombre maximal d'items à afficher par facette -->
       <str name="facet.limit">10</str>     

       <!-- Mise en surbrillance  -->
       <!--     Paramètres de base     -->
       <str name="hl">on</str>
       <str name="hl.fl">content features title name description</str>
       <str name="hl.encoder">html</str>
       <str name="hl.simple.pre">&lt;span style='background-color:yellow'&gt;&lt;b&gt;</str>
       <str name="hl.simple.post">&lt;/b&gt;&lt;/span&gt;</str>
       <!--     Paramètres de detail     -->
       <str name="hl.fragsize">800</str>
       <str name="hl.maxAnalyzedChars">1000000</str>
       <str name="hl.snippets">8</str>
       <str name="hl.snippets">8</str>
       <str name="f.title.hl.fragsize">0</str>
       <str name="f.title.hl.alternateField">title</str>
       <str name="f.name.hl.fragsize">0</str>
       <str name="f.name.hl.alternateField">name</str>
       <str name="f.content.hl.snippets">3</str>
       <str name="f.content.hl.fragsize">800</str>
       <str name="f.content.hl.alternateField">content</str>
       <str name="f.content.hl.maxAlternateFieldLength">750</str>

       <!-- analyseur de requêtes (le plus complet, edismax, est choisi)  -->
       <str name="defType">edismax</str>
        
       <!-- boosts ou poids des champs lors du classement des documents retournés -->
       <str name="qf">
          description^5.0 title^10.0 pubname^1.2
       </str>
    </lst>
  </requestHandler>
```

Après remplacer la configuration minimale pour `browse` qui vient par défaut en Solr 8.3.0 par la configuration ci-dessus, et effectuer un `RELOAD` du core ([commande](#reloadConfig)), on voit que les résultats s'affichent d'une façon plus riche, et qu'il n'est plus nécessaire de spécifier le champ de recherche dans la requête. L'image ci-dessous montre le nouvel affichage.
- On voit que la requête est maintenant `cheval`, sans besoin de spécifier le champ (pas `description:cheval`)
- Les facettes qui ont été demandées dans la configuration (p. ex. `enti_location_str`, `enti_organization_str` sont maintenant visibles à gauche. À noter que, comme c'est des facettes, il s'agit de champs de recherche *exacte* (type `string` dans le schéma), dont les contenus s'affichent tel quel, sans racinisation
- L'expression recherchée (*cheval*) est mise en surbrillance.
    - On voit également le terme *chevaux* au pluriel en surbrillance. Ceci se doit à ce que nous avons indexé le champ `description` (et autres, voir [ci-dessus](#configSchema)) comme étant du texte français, et le stemming pour le français s'applique correctement afin de trouver les documents contenant le mot au pluriel en cherchant le singulier et vice-versa

![affichage HTML configuration complète](images/browse-modif-1.png "Browse mieux configuré")

##  Interface de navigation de résultats <a name="interface"/>
 
### Utilité <a name="interfaceUtilite"/>

Sur la base de la configuration créé, nous pouvons améliorer l'expérience de navigation à travers la modification de l'interface HTML. Par exemple, nous pouvons ajouter deux colonnes de facettes, comme dans l'image ci-dessous.
 - La colonne de gauche contient les facettes provenant d'annotations obtenues par le Traitement automatique des langues (entités nommées). 
 - La colonne de droite contient des métadonnées qui étaient déjà fournies par les journaux d'où proviennent les articles (métadonnées éditoriales) : nom de la publication, date de publication.

 - Cette configuration nous permet de filtrer les résultats pour cibler certaines informations rapidement. P. ex. nous cherchont le mot *migrants* et nous pourrions ensuite restreindre les résultats grâce aux facettes aux documents où le lieu géographique *Mexique* est mentionné provenant de publications qui partagent une orientation politique donnée (p. ex. des journaux de gauche vs. de droite)

![affichage HTML configuration finale](images/browse-modif-complete.png "Browse configuration finale")


### Configuration <a name="configuration"/>

Solr fournit quelques templates par défaut à mode d'interface de recherche. Ces templates utilisent la librairie [Velocity](https://velocity.apache.org/).

Pour modifier les templates, nous pouvons les extraire de la distribution Solr, avec la commande suivante depuis la racine de votre instance Solr:

```shell
unzip dist/solr-velocity-*.jar velocity/*.vm -d server/solr/presse/conf/
```

Comme l'indique la commande, les templates sont à placer dans un dossier nommé `velocity` dans le dossier de configuration de l'index. Ça donne `server/solr/presse/conf/velocity` donc, à partir de la racine de l'instance Solr.

Des templates déjà configurés pour obtenir l'affichage avec deux colonnes de facettes ci-dessus se trouvent sur [GitLab](./01_recherche_lexicale/templates/velocity_modifie)

Vous pouvez prendre le dossier `velocity_modifie` au complet et le copier dans `server/solr/presse/conf`, en changeant le nom du dossier à `velocity`.

En rafraichissant la page ça devrait donner l'affichage ci-dessus, avec deux colonnes de facettes.

Or, il serait intéressant d'avoir une connaissance de base sur le fonctionnement des templates, ce qui est le sujet de la section suivante

### Templates Velocity <a name="templatesVelocity"/>

Comme décrit, l'emplacement des templates est à l'intérieur du dossier `conf` du core, dans un dossier nommé `velocity`. 

<img src="images/core_dir_velocity_in_place.png" alt="arborescence d'un core avec templates velocity" width="250"/>

Les templates principaux (et pertinents pour ce TP) sont les suivants:

- `browse.vm` : Template de base
    - Contient le formulaire de requête
    - Contrôle la mise en page générale (composantes à afficher) : colonnes de facettes, barre de pagination, entête et pied de la page
- `hit.vm` : Affichage de chaque document de la liste de résultats
- `facets.vm` : Comment traiter les facettes. 
- `layout.vm` : Nous pourrons enlever le logo Solr et/ou afficher notre propre logo au lieux grâce à ce template


Pour modifier les styles, le **CSS** est dand le template `head.vm`

Note: Dans les templates, `#` introduce du code Velocity, et `##` introduit on commentaire.

#### Modifications principales

Une bonne façon de se renseigner sur la modification des templates serait de comparer les templates par défaut et les templates modifiés avec un outil de diff. Un dossier sur [GitLab](./01_recherche_lexicale/templates) contient les deux jeux de templates. L'image montre le diff à niveau du dossier :

<!--![diff dir velocity](images/diff_dir_templates.png "Diff sur répertoire velocity_original et velocity_modifie")-->

<img src="images/diff_dir_templates.png" alt="Diff sur répertoire velocity_original et velocity_modifie" width="600">


Quelques commentaires sur les modifications effectuées (voir aussi les captures d'écran du diff) :
- Dans `browse.vm` :
    - À part l'ajout d'un bouton *reset* qui vide le formulaire, le plus important est l'ajout d'un élément *div* pour accueillir les facettes à droite (le CSS a également été modifié pour formater ce div)
<img src="images/diff-browse.png" alt="Diff sur browse.vm entre velocity_original et velocity_modifie" width="700"/>

- Dans `hit.vm` :
    - Les champs à afficher ont été restreints avec un tableau les mentionnant (ligne 5)
    - Un URL cliquable pour le document a été ajouté (ligne 24, pas visible dans l'image)
<img src="images/hit_modif.png" alt="Diff sur hit.vm entre velocity_original et velocity_modifie"/>

- Dans `facets.vm`:
    - Les facettes à afficher ont été restreintes à `enti_location` et `enti_organization` (ligne 14)
    - Un tableau (ligne 6) associe le nom de la facette dans l'index avec le nom qui sera utilisé pour le titre de niveau h3 qui précède l'affichage de la facette. 
<img src="images/diff_facets.png" alt="Diff sur facets.vm entre velocity_original et velocity_modifie"/>


- Ajout de `facets-2.vm` :
    - Même idée que `facets.vm` : Les facettes à afficher à droite ont été choisies, et un tableau associe le nom de la facette dans l'index avec le nom qui servira d'entête pour son affichage
- Dans `layout.vm` :
    - Le logo Solr a été enlevé
- Dans `head.vm` :
    - Ce module contient un CSS interne, qui a été modifié pour s'adapter aux changements dans les autres modules


#### Autres modifications

##### Changement du nom affiché pour chaque champ

Une autre modification possible dans `hit.vm` consiste à changer le nom avec lequel les champs sont affichés. Nous pouvons utiliser le même principe de tableau associatif déjà exploité pour renommer les entêtes de chaque facette dans `facets.vm` et `facets-2.vm`. Dans le diff ci-dessous nous voyons la modification à faire dans `hit.vm` de façon à ce que le contenu du champ `description` s'affiche précéde de **text**, le champ `pdate` précédé de **date** et le champ `pubname` précédé de **source**, plutôt que précédé du nom littéral du champ dans l'index.

  - Un tableau associatif (nommé dans ce cas `$fieldRenamer` mais tout autre nom est possible) a été déclaré pour relier les noms des champs du schéma aux noms qui seront utilisés pour l'affichage dans `hit.vm`. 
  - On test avec `containsKey()` si le nom de chaque champ du document est présent comme clé dans le tableau, dans lequel cas la valeur reliée sera affichée. Au contraire, on laisse le nom du champ pareil comme dans l'index. 

<img src="images/diff_renommage_champs_hit_velocity.png" alt="Diff sur hit.vm entre velocity_original et velocity_modifie avec tableau associatif pour renommage des champs avant affichage"/>

Le panneau central (celui des documents) dans l'image ci-dessous montre le nouvel affichage avec les noms de champs remplacés.

<img src="images/champs_hit_renommes.png" alt="Interface une fois les noms des champs des documents renommés avant affichage dans l'index"/>

##### Accès aux valeurs d'un champ quelconque du document depuis les templates

Pour avoir accès à un des champs du document depuis les templates, on peut utiliser la fonction `$doc.getFieldValue()`.

Par exemple, s'il y a un champ nommé `url` dans les documents qui contient l'URL pour la source des documents, nous pouvons avoir accès à la valeur du champ avec `$doc.getFieldValue('url')[0]`. Ensuite nous pouvons profiter de cette valeur pour générer une lien externe vers la source (`<a href="$doc.getFieldValue('url')[0]" target="_blank">$doc.getFieldValue('url')[0]</a>`).
