# TP - Recherche d'information partie 2 : Assistant avec API Mistral et *function calling*

1. [Introduction](#intro)
1. [Prérequis](#prerequis)
    1. [Clé API Mistral](#cle-api-mistral)
    1. [Clé API Méteo](#cle-api-meteo)
1. [Tâche à faire](#tache-a-faire)
1. [Autres informations](#autres-informations)
    1. [Création d'un client Mistral pour génération de texte](#client-mistral)
    1. [Function calling](#function-calling)
    1. [L'API d'Open Library](#api-open-library)

<a name="intro"/></a>

## Introduction 

Dans cette partie du TP il s'agit de travailler avec la fonctionnalité [*function calling*](https://docs.mistral.ai/capabilities/function_calling/) de l'API de Mistral. Cette fonctionnalité permet de définir des fonctions qui peuvent être appelées par l'API de Mistral lors d'un dialogue avec un des modèles. Quand le texte saisi par l'utilistaeur·trice contient des informations pertinentes pour appeler une de ces fonctions, le besoin de déclencher la fonction et les valeurs des arguments à passer sont détectés par le modèle. L'API appelle ensuite la fonction.

Les informations retournées par la fonction peuvent être utilisées dans des tours de parole subséquents pour répondre à de nouvelles questions de l'utilisateur·trice. Nous combinons ainsi les capacités d'analyse et génération d'un LLM avec des informations externes qui ne pouvaient pas être disponibles lors de l'entraînement du modèle. P. ex. le modèle peut avoir une fonction pour détecter une demande sur la météo actuelle dans une ville, détecter la ville et l'échelle (Celsius ou Fahrenheit) et retourner la température actuelle. Ensuite il peut émettre par exemple un avis sur commment s'habiller en fonction des informations sur la météo.

<a name="prerequis"/></a>

## Prérequis

<a name="cle-api-mistral"/></a>

### Clé API Mistral

Nous aurons besoin d'une clé (gratuit) pour utiliser Mistral. Pour obtenir une clé, il faut s'inscrire sur [https://console.mistral.ai/](https://console.mistral.ai/) et choisir le plan *Experiment*. En ce faisant, il ne sera pas nécessaire de fournir des informations de paiement. La procédure est gratuite et demande la vérification d'une adresse mail, ainsi que par SMS la vérification d'un numéro de téléphone.

En cas de besoin de détails, la procédure actuelle (mars 2025) pour créer une clé est décrite dans [ce document](02_assistant_mistral/doc/creation_cle_api_mistral.md).

Après création de la clé, vous pouvez l'ajouter comme variable d'environnement, p. ex. avec un nom de variable comme `MISTRAL_API_KEY`. Vous pouvez aussi la saisir directement dans le script, mais ce n'est pas une bonne pratique si le code va être partagé.

Il faut donner la clé lors de l'instantation du client Mistral. P. ex., si la clé est dans une variable d'environnement `MISTRAL_API_KEY` :

```python
client = Mistral(api_key=os.getenv("MISTRAL_API_KEY"))
```

<a name="cle-api-meteo"/></a>

### Clé API méteo

Il est aussi nécessaire d'avoir une clé gratuite pour l'une des deux API utilisées dans le *function calling*, [Weather API](https://www.weatherapi.com/), pour obtenir des informations sur la météo. La clé est gratuite et ne demande pas d'informations de paiement. Pour l'obtenir, on peut aller sur *Signup* et remplir les informations.

La deuxième API ([Open Library](https://openlibrary.org/dev/docs/api/search)) ne demande pas de clé.

<a name="tache-a-faire"></a>
## Tâche à faire

### Spécifications

Le client Mistral dans `assistant_mistral_a_completer.py` permet d'interagir avec un modèle Mistral de façon libre, jusqu'à la saisie de `###` par l'utilisateur·trice. Le modèle peut aussi répondre à des questions sur la météo actuelle dans une ville. Pour cela, il utilise la fonction `get_weather`. Cette fonction appelle l'API de [Weather API](https://www.weatherapi.com/) pour obtenir les informations sur la météo. 

**La tâche consiste à ajouter du code pour permettre à l'assistant de réagir à une requête utilisateur qui exploite des informations disponibles sur Open Library : obtenir des ouvrages par un auteur·trice dans une langue donnée ; une solution pour cette tâche est disponible sur `assistant_mistral.py`. Le modèle détectera que le tour de parole consiste à une demande d'information sur les ouvrages par un auteur·trice, dans une langue donnée, et appellera `get_openlibrary_data` avec les paramètres nécessaires.**

Vous pouvez compléter `assistant_mistral_a_completer.py` ou faire votre propre code à partir de zéro.

Si vous travaillez à partir du script `assistant_mistral_a_completer.py`, il faut compléter aux endroits suivants :

- La fonction `get_openlibrary_data`
- La définition de la gestion de la fonction `get_openlibrary_data` dans la liste de ressources définies dans la variable `tools`.
- La gestion de la réponse de `get_openlibrary_data` dans la boucle principale de l'assistant : Comment gérer la réponse de la fonction pour intégrer ses résultats à la conversation, une fos que le modèle a détecté la demande de l'utilisateur·trice et appelé la fonction.

### Exemple du comportement souhaité

Un exemple du type de conversation que cet assistant permet (qui correspond à la solution dans `assistant_mistral.py`) est ci-dessous. Les réponses sont précédées des mots suivants selon si le *function calling* s'est appliqué ou pas:

- Assistant (weather)
- Assisatant (openlibrary)
- Assistant (no function call)

```
In [1]: %run assistant_mistral.py

This is the beginning of your chat with AI. [To exit, send "###".]

You: J'aimerais lire des ouvrages par Virginie Despentes en français

Assistant (openlibrary): Virginie Despentes (lang fre):
- King Kong théorie (2006)
- Baise-moi (1999)
- Vernon Subutex 1 (2015)
- Cher connard (2022)
- Apocalypse Bébé (2010)
- Teen Spirit (2002)
- Bye bye Blondie (2004)
- Les Chiennes savantes (1996)
- Les jolies choses (1998)
- Mordre au travers (2001)
- Mordre Au Travers (1998)
- Dix (1997)

You: What's the weather like in Montréal?

Assistant (weather): Montréal: 4.2 celsius.

You: Comment m'habiller là-bas?

Assistant (no function call): Il fait 4.2 degrés Celsius à Montréal.
Je vous recommande
de porter des vêtements chauds, comme un manteau,
un pull, un pantalon et des bottes.
N'oubliez pas de porter des gants
et une écharpe pour vous protéger du froid.

You: How can I run Python one-liners on the command line?

Assistant (no function call): To run Python one-liners 
on the command line, you can use the Python interpreter  
directly from the terminal or command prompt. 
Here are a few methods to do this:

[...]
```

La conversation dans l'exemple mélange l'anglais et le français, mais il est peut-être plus simple d'utiliser une seule langue pour ne pas avoir à gérer les codes de langue requis par l'API de Open Library quand on lui demande les ouvrages dans une langue concrète, au moins que le modèle Mistral fournit déjà spontanément des codes de langue (ISO-639-2B) pour la langue mentionnée par l'utilisateur·trice. En utilisant une seule langue, le code de langue peut être donné explicitement dans le script au lieu d'obtenu dynamiquement. 

### Function calling vs. Agents

Dans ce TP nous avons utilisé le function calling vu la tâche plutôt simple. Pour des scénarios complexes, il peut être plus adapté d'utiliser les *Agents* plutôt que le *function calling*.

Les *Agents* agissent comme des systèmes autonomes qui s'occupent d'une seule tâche. Nous pouvons combiner des agents pour résoudre une tâche complexe, en créant un agent spécialisé pour chaque étape de la tâche. Les agents sont décrits sur https://docs.mistral.ai/capabilities/agents/. Le notebook d'exemple sur https://github.com/mistralai/cookbook/blob/main/mistral/agents/analytical_agent_workflow.ipynb montre leur potentiel.

<a name="autres-informations"></a>

## Autres informations

<a name="client-mistral"></a>

### Création d'un client Mistral pour génération de texte :

Un client pour la génération de texte étant donné un prompt utilise l'endpoint `chat` de l'API de Mistral :

```python
import os
from mistralai import Mistral

client = Mistral(api_key=os.getenv("MISTRAL_API_KEY"))
model = "mistral-small-latest"


conversation = [
   {"role": "system", "content": "You are a helpful assistant."},
   {"role": "user", "content": "Where is the answer blowing?"}
]

response = client.chat.complete(
   model=model,
   messages=conversation
)

print(f"\nAssistant: {response.choices[0].message.content}")
```

Pour créer un client Mistral, il faut donner la clé API comme argument, ici la clé était définie comme variable d'environnement.

La variable nommé ici `conversation` est une liste de dictionnaires qui sert à donner les différentes parties du prompt. La question posée au modèle est la valeur pour la clé `content` dans le dictionnaire dont la valeur de la clé `role` est `user`. Ici la question est *Where is the answer blowing?*, le modèle va générer une réponse à cette question. 

Il est optionnel d'inclure comme partie initiale du prompt une clé `role` avec la valeur `system` pour donner des informations au modèle sur la nature du système envisagé. Un prompt neutre dans ce sens est quelque chose comme *You are a helpful assistant.*, mais on pourrait varier cette chaîne pour essayer d'obtenir un comportement différent du modèle. On peut donner là des informations globales qu'il n'y aurait pas besoin de répéter pour chaque prompt. 

La réponse est demandée au modèle dans la variable nommée ici `response`. On appelle la méthode `complete` de `client.chat`, en donnant comme arguments le modèle à appeler (ici un modèle basique nommé `mistral-small-latest`, comme la valeur de l'argument `model`) et le prompt complet comme la valeur de l'argument `messages`. La réponse générée par le modèle se trouvera dans `response.choices[0].message.content`.

```python
response = client.chat.complete(
   model=model,
   messages=conversation
)
```

Nous pouvons généraliser ce code pour créer un assistant qui lit le texte saisi par l'utilisateur·trice en boucle et génère des réponses, jusqu'à ce que l'utilisateur·trice écrit un texte de sortie. Ceci a été fait dans le script [`assistant_mistral_a_completer.py`](02_assistant_mistral/assistant_mistral_a_completer.py), avec `###` comme texte de sortie. 

<a name="function-calling"></a>

### Function calling

Cette fonctionalité est décrite sur https://docs.mistral.ai/capabilities/function_calling/. Elle nous permet de définir des fonctions qui peuvent être appelées par l'API de Mistral lors d'un dialogue avec un modèle. Quand le texte saisi par l'utilistaeur·trice contient des informations pertinentes pour appeler une des fonctions que nous avons définies, le besoin de déclencher la fonction et les valeurs des arguments à utiliser dans l'appel sont détectés par le modèle. L'API appelle ensuite la fonction.

Un exemple est disponible dans [`assistant_mistral_a_completer.py`](02_assistant_mistral/assistant_mistral_a_completer.py) pour appeler l'API Wheather API.

Utiliser le function calling demande d'ajouter un argument `tools` lors de l'appel à la méthode `complete` de `client.chat`.

```python
response = client.chat.complete(
    model="mistral-small-latest",
    messages=conversation,
    tools=tools
)
```

L'argument `tools` fait référence à une liste de dictionnaires, chaque dictionnaire définissant une fonction à appeler. Dans le cas de `get_weather`, le dictionnaire est :

```python
{
   "type": "function",
   "function": {
       "name": "get_weather",
       "parameters": {
           "type": "object",
           "properties": {
               "location": {"type": "string"},
               "unit": {"type": "string"},
           },
           "required": ["location", "unit"],
           "additionalProperties": False,
       },
   },
}
```

Sur la base de cela, vous pouvez définir une fonction `get_openlibrary_data` et compléter l'assistant dans [`assistant_mistral_a_completer.py`](02_assistant_mistral/assistant_mistral_a_completer.py) pour répondre à la tâche décrite dans [*Tâche à faire* ci-dessus](#tache-a-faire). Plus de détails sur le *function calling* sur https://docs.mistral.ai/capabilities/function_calling/.

<a name="api-open-library"></a>

### L'API d'Open Library

Comme décrit sur leur [site](https://openlibrary.org), « Open Library est un catalogue de bibliothèque ouvert et modifiable, qui cherche à créer une page web pour chaque livre qui ait jamais été publié ». C'est une initiative de l'Internet Archive (qui gère aussi le Wayback Machine).

L'API d'Open Library permet de faire des recherches dans le catalogue. Les données sont stockées dans Solr, et les commandes de l'API ainsi que le format de la réponse en JSON sont fondés sur la syntaxe Solr introduite dans la première partie du TP.

Pour la documentation de l'API, voir https://openlibrary.org/dev/docs/api et plus concrètement l'API *Book Search* sur https://openlibrary.org/dev/docs/api/search, qui peut être utilisée pour la [tâche du TP](#tache-à-faire).

Les détails sont disponibles sur [https://openlibrary.org/dev/docs/api/search](https://openlibrary.org/dev/docs/api/search).